@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.file.storage.mail;
 
+import static com.openexchange.java.Autoboxing.L;
 import java.util.Date;
 import java.util.EnumSet;
 import java.util.List;
@@ -33,7 +34,10 @@ import com.openexchange.file.storage.FileStorageFileAccess;
 import com.openexchange.file.storage.FileStorageFolder;
 import com.openexchange.file.storage.mail.osgi.Services;
 import com.openexchange.java.Strings;
+import com.openexchange.mail.mime.ContentDisposition;
+import com.openexchange.mail.mime.ContentType;
 import com.openexchange.mail.mime.MessageHeaders;
+import com.openexchange.mail.mime.MimeType2ExtMap;
 import com.openexchange.mail.mime.MimeTypes;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mime.MimeTypeMap;
@@ -47,6 +51,8 @@ import com.sun.mail.imap.IMAPMessage;
  */
 public final class MailDriveFile extends DefaultFile {
 
+    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailDriveFile.class);
+
     /**
      * Parses an instance of <code>MailDriveFile</code> from specified IMAP message.
      *
@@ -134,10 +140,7 @@ public final class MailDriveFile extends DefaultFile {
     private MailDriveFile parseMessage(IMAPMessage message, List<Field> fields) throws MessagingException, OXException {
         if (null != message) {
             try {
-                String name = MimeMessageUtility.getSubject(message);
-                if (Strings.isEmpty(name)) {
-                    return null;
-                }
+                String name = determineFileNameFor(message);
                 setTitle(name);
                 setFileName(name);
                 final Set<Field> set = null == fields || fields.isEmpty() ? EnumSet.allOf(Field.class) : EnumSet.copyOf(fields);
@@ -209,4 +212,108 @@ public final class MailDriveFile extends DefaultFile {
         return this;
     }
 
+    /**
+     * Determines the file name to assume for specified IMAP message.
+     *
+     * @param message The IMAP message
+     * @return The file name
+     * @throws MessagingException If a suitable file name cannot be determined
+     */
+    private String determineFileNameFor(IMAPMessage message) throws MessagingException {
+        String name = MimeMessageUtility.getSubject(message);
+        if (Strings.isEmpty(name)) {
+            name = extractFileNameFromContentDisposition(message);
+            if (Strings.isEmpty(name)) {
+                name = extractFileNameFromContentType(message);
+            }
+        }
+
+        if (Strings.isEmpty(name)) {
+            // File name unknown; neither by Subject, nor by Content-Type/Content-Disposition headers
+            ContentType contentType = getContentTypeFromOrElse(message, ContentType.APPLICATION_OCTETSTREAM_CONTENT_TYPE);
+            LOG.warn("Missing file name in IMAP message {} in folder {}. Trying to deduce from MIME type: {}", L(message.getUID()), getFolderId(), contentType);
+            List<String> exts = MimeType2ExtMap.getFileExtensions(Strings.asciiLowerCase(contentType.getBaseType()));
+            name = new StringBuilder(16).append("file.").append(exts == null ? "dat" : exts.get(0)).toString();
+        }
+        return name;
+    }
+
+    /**
+     * Extracts possible file name from <code>"filename"</code> parameter of <code>"Content-Disposition"</code> header.
+     * <pre>
+     *  Content-Disposition: attachment; filename=letter.pdf
+     * </pre>
+     *
+     * @param message The IMAP message to acquire the header from
+     * @return The file name as specified by <code>"filename"</code> parameter of <code>"Content-Disposition"</code> header
+     */
+    private String extractFileNameFromContentDisposition(IMAPMessage message) {
+        ContentDisposition cd = getContentDispositionFromOrElse(message, null);
+        return cd == null ? null : cd.getFilenameParameter();
+    }
+
+    /**
+     * Gets the <code>"Content-Disposition"</code> header from given IMAP message if such a header is present (and parseable); otherwise
+     * given <code>defaultContentDisposition</code> is returned.
+     *
+     * @param message The IMAP message to get <code>"Content-Disposition"</code> header from
+     * @param defaultContentDisposition The default disposition to return on absence or parsing failure
+     * @return The disposition or <code>defaultContentDisposition</code>
+     */
+    private ContentDisposition getContentDispositionFromOrElse(IMAPMessage message, ContentDisposition defaultContentDisposition) {
+        String hdrContentDisposition = null;
+        try {
+            hdrContentDisposition = message.getHeader(MessageHeaders.HDR_CONTENT_DISPOSITION, null);
+            if (Strings.isNotEmpty(hdrContentDisposition)) {
+                return new ContentDisposition(hdrContentDisposition);
+            }
+        } catch (Exception e) {
+            if (hdrContentDisposition == null) {
+                LOG.warn("Failed to acquire {} header from IMAP message {} in folder {}", MessageHeaders.HDR_CONTENT_TYPE, L(message.getUID()), getFolderId(), e);
+            } else {
+                LOG.warn("Failed to parse {} header from IMAP message {} in folder {}: {}", MessageHeaders.HDR_CONTENT_TYPE, L(message.getUID()), getFolderId(), hdrContentDisposition, e);
+            }
+        }
+        return defaultContentDisposition;
+    }
+
+    /**
+     * Extracts possible file name from <code>"name"</code> parameter of <code>"Content-Type"</code> header.
+     * <pre>
+     *  Content-Type: application/pdf; name=letter.pdf
+     * </pre>
+     *
+     * @param message The IMAP message to acquire the header from
+     * @return The file name as specified by <code>"name"</code> parameter of <code>"Content-Type"</code> header
+     */
+    private String extractFileNameFromContentType(IMAPMessage message) {
+        ContentType ct = getContentTypeFromOrElse(message, null);
+        return ct == null ? null : ct.getNameParameter();
+    }
+
+    /**
+     * Gets the <code>"Content-Type"</code> header from given IMAP message if such a header is present (and parseable); otherwise given
+     * <code>defaultContentType</code> is returned.
+     *
+     * @param message The IMAP message to get <code>"Content-Type"</code> header from
+     * @param defaultContentType The default MIME type to return on absence or parsing failure
+     * @return The MIME type or <code>defaultContentType</code>
+     */
+    private ContentType getContentTypeFromOrElse(IMAPMessage message, ContentType defaultContentType) {
+        String hdrContentType = null;
+        try {
+            hdrContentType = message.getHeader(MessageHeaders.HDR_CONTENT_TYPE, null);
+            if (Strings.isNotEmpty(hdrContentType)) {
+                return new ContentType(hdrContentType);
+            }
+        } catch (Exception e) {
+            if (hdrContentType == null) {
+                LOG.warn("Failed to acquire {} header from IMAP message {} in folder {}", MessageHeaders.HDR_CONTENT_TYPE, L(message.getUID()), getFolderId(), e);
+            } else {
+                LOG.warn("Failed to parse {} header from IMAP message {} in folder {}: {}", MessageHeaders.HDR_CONTENT_TYPE, L(message.getUID()), getFolderId(), hdrContentType, e);
+            }
+        }
+        return defaultContentType;
+    }
+
 }
